﻿using System;
using System.Linq;
using System.Data;
using System.Data.Common;

namespace CrmDemoCodeFirst {

  class Program {

    static void Main(string[] args) {

      // We have turned on OracleMonitor to view executed DDL и DML statements in Devart dbMonitor application
      Devart.Data.Oracle.OracleMonitor monitor = new Devart.Data.Oracle.OracleMonitor() { IsActive = true };

      //--------------------------------------------------------------
      // You use the capability for configuring the behavior of the EF-provider:
      Devart.Data.Oracle.Entity.Configuration.OracleEntityProviderConfig config = Devart.Data.Oracle.Entity.Configuration.OracleEntityProviderConfig.Instance;
      // Now, you switch off schema name generation while generating DDL scripts and DML:
      config.Workarounds.IgnoreSchemaName = true;
      //--------------------------------------------------------------

      //--------------------------------------------------------------
      // You can set up a connection string for DbContext in different ways.
      // It can be placed into the app.config (web.config) file.
      // The connection string name must be identical to the DbContext descendant name.
      // <add name="MyOracleContext" connectionString="Data Source=ora1020;User Id=test;Password=test;" providerName="Devart.Data.Oracle" />
      // After that, you create a context instance, while a connection string is enabled automatically:
      // MyDbContext context = new MyDbContext();
      //---------------------------------------------------------------

      //--------------------------------------------------------------
      // And now it is possible to create an instance of the provider-specific connection 
      // and send it to the context constructor, like we did in this application.
      // That allows us to use the StateChange connection event to change
      // the Oracle current schema on its occurrence.
      // Thus, we can connect as one user and work on a schema owned by another user.
      //--------------------------------------------------------------
      DbConnection con = new Devart.Data.Oracle.OracleConnection("user id=scott;password=tiger;server=ORCL1020;persist security info=True");
      con.StateChange += new StateChangeEventHandler(Connection_StateChange);
      //--------------------------------------------------------------

      /*--------------------------------------------------------------
        Let's create MyDbContext and execute a database query.
        Depending on selected database initialization strategу,
        database tables can be deleted/added, and filled with source data.
      ---------------------------------------------------------------*/

      MyDbContext context = new MyOracleContext(con);
      Queries queries = new Queries();

      Console.WriteLine("dotConnect for Oracle : Entity Framework 6 Code-First sample");
      Console.WriteLine();

      queries.GetCategoriesByPrice(context);
      Console.WriteLine();

      queries.GetOrdersSince2007(context);
      Console.ReadKey();
    }

    // On connection opening, we change the current schema to "TEST":
    static void Connection_StateChange(object sender, StateChangeEventArgs e) {

      if (e.CurrentState == ConnectionState.Open) {
        DbConnection connection = (DbConnection)sender;
        connection.ChangeDatabase("TEST");
      }
    }
  }




}